﻿using Prism.Ioc;
using Borepin.PageModel;
using Borepin.Page;
using Xamarin.Forms;
using Borepin.Dialog;
using Borepin.DialogModel;
using Borepin.Service.BFFH;
using Prism;
using Borepin.Page.SetUpProcess;
using Borepin.PageModel.SetUpProcess;
using Borepin.Page.AddServerProcess;
using Borepin.PageModel.AddServerProcess;
using System;

namespace Borepin
{
    public partial class App
    {
        public App(IPlatformInitializer platformInitializer) : base(platformInitializer)
        {
            
        }

        protected override async void OnInitialized()
        {
            InitializeComponent();

            await NavigationService.NavigateAsync(new Uri("https://borepin.fab-access.org/StartPage")).ConfigureAwait(false);
        }

        protected override void RegisterTypes(IContainerRegistry containerRegistry)
        {
            #region Register Basic Navigation
            containerRegistry.RegisterForNavigation<MainPage, MainPageModel>();
            containerRegistry.RegisterForNavigation<NavigationPage>();

            containerRegistry.RegisterForNavigation<StartPage, StartPageModel>();

            containerRegistry.RegisterForNavigation<MachinePage, MachinePageModel>();
            containerRegistry.RegisterForNavigation<SettingsPage>();
            containerRegistry.RegisterForNavigation<MachineListPage, MachineListPageModel>();
            containerRegistry.RegisterForNavigation<ServerListPage, ServerListPageModel>();
            containerRegistry.RegisterForNavigation<ServerPage, ServerPageModel>();
            containerRegistry.RegisterForNavigation<ScannerPage>();
            #endregion

            #region Register Sequence Navigation
            containerRegistry.RegisterForNavigation<WelcomePage, WelcomePageModel>("SetUpProcess_WelcomePage");
            //containerRegistry.RegisterForNavigation<ScanPage, ScanPageModel>("SetUpProcess_ScanPage");

            containerRegistry.RegisterForNavigation<AuthPlainPage, AuthPlainPageModel>("AddServerProcess_AuthPlainPage");
            containerRegistry.RegisterForNavigation<SelectServerPage, SelectServerPageModel>("AddServerProcess_SelectServerPage");
            containerRegistry.RegisterForNavigation<ChooseAuthTypePage, ChooseAuthTypePageModel>("AddServerProcess_ChooseAuthTypePage");
            #endregion

            #region Register Dialog
            containerRegistry.RegisterDialog<ConfirmDialog, ConfirmDialogModel>();
            containerRegistry.RegisterDialog<ScanDialog, ScanDialogModel>();
            #endregion

            #region Register Service
            containerRegistry.RegisterSingleton<IBFFHService, BFFHService>();
            #endregion
        }
    }
}
