﻿using Prism.Mvvm;
using Prism.Navigation;
using System.Threading.Tasks;

namespace Borepin.Base
{
    /// <summary>
    /// Base for all MVVM Based PageModels
    /// </summary>
    public abstract class PageModelBase : BindableBase, INavigationAware
    {
        #region Private Fields
        protected readonly INavigationService _NavigationService;

        protected PageModelBase(INavigationService navigationService)
        {
            _NavigationService = navigationService;
        }
        #endregion

        #region Fields
        /// <summary>
        /// PageModel is Busy
        /// </summary>
        private bool _IsBusy = true;
        public bool IsBusy
        {
            get => _IsBusy;
            set => SetProperty(ref _IsBusy, value);
        }
        #endregion

        #region Data
        /// <summary>
        /// Load Data async
        /// </summary>
        /// <returns></returns>
        public abstract Task LoadData();
        #endregion

        #region INavigationAware
        public abstract void OnNavigatedTo(INavigationParameters parameters);
        public abstract void OnNavigatedFrom(INavigationParameters parameters);
        #endregion
    }
}