﻿using Plugin.Multilingual;
using System;
using System.Reflection;
using System.Resources;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace Borepin.Helpers
{
    [ContentProperty("Text")]
    public class TranslateExtension : IMarkupExtension
    {
        const string _ResourceId = "Borepin.Resources.Text.TextResource";

        static readonly Lazy<ResourceManager> _Resmgr = new Lazy<ResourceManager>(() => new ResourceManager(_ResourceId, typeof(TranslateExtension).GetTypeInfo().Assembly));

        public string Text { get; set; }

        public object ProvideValue(IServiceProvider serviceProvider)
        {
            if (Text == null)
            {
                return "";
            }

            System.Globalization.CultureInfo ci = CrossMultilingual.Current.CurrentCultureInfo;

            string translation = _Resmgr.Value.GetString(Text, ci);

            if (translation == null)
            {

#if DEBUG
                //throw new ArgumentException(
                //    String.Format("Key '{0}' was not found in resources '{1}' for culture '{2}'.", Text, ResourceId, ci.Name),
                //    "Text");
                translation = "!MISSING TEXT!";
#else
                translation = Text; // returns the key, which GETS DISPLAYED TO THE USER
#endif
            }
            return translation;
        }
    }
}
