﻿using System;
using System.Collections.Generic;

namespace Borepin.Model
{
    /// <summary>
    /// Class to contain all Information about a Connection
    /// </summary>
    public class Connection
    {
        #region Constructors
        public Connection()
        {
            Address = new Uri("http://127.0.0.1:59661");
        }

        public Connection(Connection connection)
        {
            ConnectionTyp = connection.ConnectionTyp;
            AuthenticationTyp = connection.AuthenticationTyp;
            Address = connection.Address;
            Username = connection.Username;
            LastTime = connection.LastTime;
        }
        #endregion

        #region Members
        /// <summary>
        /// Type of Connection
        /// </summary>
        public ConnectionTyp ConnectionTyp { get; set; } = ConnectionTyp.SINGLE;

        /// <summary>
        /// Type of Authentication
        /// </summary>
        public AuthenticationTyp AuthenticationTyp { get; set; } = AuthenticationTyp.PLAIN;

        /// <summary>
        /// Address to Host
        /// </summary>
        public Uri Address { get; set; }

        /// <summary>
        /// Username for Connection
        /// </summary>
        public string Username { get; set; } = "";

        /// <summary>
        /// Last Timestamp connection was successfully established
        /// </summary>
        public DateTime LastTime { get; set; }
        #endregion

        #region Methods
        #region Equals and HashCode
        public override bool Equals(object obj)
        {
            return obj is Connection connection &&
                   EqualityComparer<Uri>.Default.Equals(Address, connection.Address) &&
                   string.Equals(Username, connection.Username, StringComparison.Ordinal) &&
                   ConnectionTyp == connection.ConnectionTyp &&
                   AuthenticationTyp == connection.AuthenticationTyp;
        }

        public override int GetHashCode()
        {
            int hashCode = -904541792;
            hashCode = hashCode * -1521134295 + EqualityComparer<Uri>.Default.GetHashCode(Address);
            hashCode = hashCode * -1521134295 + LastTime.GetHashCode();
            hashCode = hashCode * -1521134295 + StringComparer.Ordinal.GetHashCode(Username);
            hashCode = hashCode * -1521134295 + ConnectionTyp.GetHashCode();
            hashCode = hashCode * -1521134295 + AuthenticationTyp.GetHashCode();
            return hashCode;
        }
        #endregion
        #endregion
    }
}