﻿using Borepin.Base;
using Borepin.Model;
using Borepin.Service.BFFH;
using Prism.AppModel;
using Prism.Navigation;
using Prism.Services;
using Prism.Services.Dialogs;
using System;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace Borepin.PageModel.AddServerProcess
{
    public class SelectServerPageModel : PageModelBase, IPageLifecycleAware
    {
        #region Private Fields
        private readonly IBFFHService _BFFHService;
        private readonly IPageDialogService _PageDialogService;
        private readonly IDialogService _DialogService;

        private Connection _Connection;
        private bool _Scanned;
        #endregion

        #region Constructors
        public SelectServerPageModel(INavigationService navigationService, IBFFHService bffhService, IPageDialogService pageDialogService, IDialogService dialogService) : base(navigationService)
        {
            _BFFHService = bffhService;
            _PageDialogService = pageDialogService;
            _DialogService = dialogService;

            ConnectToServerCommand = new Command(async ()=> await ConnectToServerExecute().ConfigureAwait(false));
            DetectLocalServerCommand = new Command(DetectHostCommandExecute);
            ScanCodeCommand = new Command(ScanCodeCommandExecute);
        }
        #endregion

        #region LoadData
        public override Task LoadData()
        {
            Server = string.Format(CultureInfo.InvariantCulture, "{0}:{1}", _Connection.Address.Host, _Connection.Address.Port);

            IsBusy = false;
            return Task.CompletedTask;
        }
        #endregion

        #region Fields
        private string _Server;
        public string Server
        {
            get => _Server;
            set => SetProperty(ref _Server, value);
        }
        #endregion

        #region Commands
        private ICommand _ConnectToServerCommand;
        public ICommand ConnectToServerCommand
        {
            get => _ConnectToServerCommand;
            set => SetProperty(ref _ConnectToServerCommand, value);
        }
        public async Task ConnectToServerExecute()
        {
            IsBusy = true;

            try
            {
                string server_address = "";
                if(Regex.IsMatch(Server, "([a-zA-Z]{2,20}):\\/\\/", RegexOptions.IgnoreCase | RegexOptions.ExplicitCapture, TimeSpan.FromSeconds(1)))
                {
                    server_address = Server;
                }
                else
                {
                    server_address = "http://" + Server;
                }
                
                UriBuilder builder = new UriBuilder(server_address);
                if (builder.Port == 80)
                {
                    builder.Port = 59661;
                }

                _Connection = new Connection()
                {
                    Address = builder.Uri,
                };
            }
            catch (UriFormatException)
            {
                await _PageDialogService.DisplayAlertAsync("Connection failed", "Server address is invaild.", "Ok").ConfigureAwait(false);

                IsBusy = false;
                return;
            }

            if (!await _BFFHService.TestConnection(_Connection).ConfigureAwait(true))
            {
                await _PageDialogService.DisplayAlertAsync("Connection failed", "Unable to connect to server.", "Ok").ConfigureAwait(false);

                IsBusy = false;
                return;
            }

            INavigationParameters parameters = new NavigationParameters()
            {
                {"instance", _Connection },
            };

            await _NavigationService.NavigateAsync("AddServerProcess_ChooseAuthTypePage", parameters).ConfigureAwait(false);
        }

        private ICommand _ScanCodeCommand;
        public ICommand ScanCodeCommand
        {
            get => _ScanCodeCommand;
            set => SetProperty(ref _ScanCodeCommand, value);
        }
        public void ScanCodeCommandExecute()
        {
            _DialogService.ShowDialog("ScanDialog", ScanCodeCommandExecute_Dialog);
        }

        public void ScanCodeCommandExecute_Dialog(IDialogResult result)
        {
            if (string.Equals(result.Parameters.GetValue<string>("result"), "scanned", StringComparison.Ordinal))
            {
                Server = result.Parameters["value"] as string;
                _Scanned = true;
            }
        }

        private ICommand _DetectLocalServerCommand;
        public ICommand DetectLocalServerCommand
        {
            get => _DetectLocalServerCommand;
            set => SetProperty(ref _DetectLocalServerCommand, value);
        }
        public void DetectHostCommandExecute()
        {
            // Get Example Server Address
            Server = "127.0.0.1:59661";
        }
        #endregion

        #region IPageLifecycleAware
        public async void OnAppearing()
        {
            if (_Scanned)
            {
                _Scanned = false;

                await ConnectToServerExecute().ConfigureAwait(false);
            }
        }

        public void OnDisappearing()
        {

        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedTo(INavigationParameters parameters)
        {
            if(_Connection == null)
            {
                if (parameters.ContainsKey("instance") && parameters["instance"] is Connection)
                {
                    _Connection = parameters["instance"] as Connection;
                }
                else
                {
                    _Connection = new Connection();
                }

                LoadData();
            }
            else
            {
                IsBusy = false;
            }
        }

        public override void OnNavigatedFrom(INavigationParameters parameters)
        {

        }
        #endregion
    }
}
