﻿using Borepin.Base;
using Prism.Commands;
using Prism.Navigation;
using System.Threading.Tasks;
using System.Windows.Input;
using FabAccessAPI.Schema;
using Borepin.Service.BFFH;
using static FabAccessAPI.Schema.MachineSystem;
using Borepin.Model;
using Prism.Services;
using Borepin.Service.BFFH.Exceptions;

namespace Borepin.PageModel
{
    public class MachinePageModel : ConnectionModelBase
    {
        #region Private Fields
        private string _ID;
        private Machine _Machine;
        #endregion

        #region Contructors
        public MachinePageModel(INavigationService navigationService, IPageDialogService pageDialogService, IBFFHService bffhService) : base(navigationService, pageDialogService, bffhService)
        {
            UseMachineCommand = new DelegateCommand(UseMachineCommandExecute);
            GiveBackMachineCommand = new DelegateCommand(GiveBackMachineCommandExecute);
            ForceFreeMachineCommand = new DelegateCommand(ForceFreeMachineCommandExecute);
        }
        #endregion

        #region Data
        public override async Task LoadData()
        {
            IsBusy = true;
            if (!await CheckConnection().ConfigureAwait(false))
            {
                IsConnected = false;

                IsBusy = false;
                return;
            }

            IMachineSystem machineSystem;
            try
            {
                machineSystem = (await _BFFHService.GetSession().ConfigureAwait(false)).MachineSystem;
            }
            catch(ReconnectingFailedException)
            {
                await _PageDialogService.DisplayAlertAsync("Connection failed", "Lost connection to server.", "Ok").ConfigureAwait(false);
                
                return;
            }

            IInfoInterface info = await machineSystem.Info().ConfigureAwait(false);

            _Machine = await info.GetMachine(_ID).ConfigureAwait(false);
            MachineItem = new MachineVisualize(_Machine);
            MachineItem.LoadData();

            IsBusy = false;
        }
        #endregion

        #region Fields
        private MachineVisualize _MachineItem;
        public MachineVisualize MachineItem
        {
            get => _MachineItem;
            set => SetProperty(ref _MachineItem, value);
        }
        #endregion

        #region Commands
        private ICommand _UseMachineCommand;

        public ICommand UseMachineCommand
        {
            get => _UseMachineCommand;
            set => SetProperty(ref _UseMachineCommand, value);
        }

        public async void UseMachineCommandExecute()
        {
            IsBusy = true;
            if (!await CheckConnection().ConfigureAwait(false))
            {
                IsConnected = false;

                IsBusy = false;
                return;
            }

            Machine.IUseInterface useInterface = _Machine.Use;

            await useInterface.Use().ConfigureAwait(false);
            await LoadData().ConfigureAwait(false);

            IsBusy = false;
        }

        private ICommand _GiveBackMachineCommand;
        public ICommand GiveBackMachineCommand
        {
            get => _GiveBackMachineCommand;
            set => SetProperty(ref _GiveBackMachineCommand, value);
        }

        public async void GiveBackMachineCommandExecute()
        {
            IsBusy = true;
            if (!await CheckConnection().ConfigureAwait(false))
            {
                IsConnected = false;

                IsBusy = false;
                return;
            }

            Machine.IInUseInterface inUseInterface = _Machine.Inuse;

            await inUseInterface.GiveBack().ConfigureAwait(false);
            await LoadData().ConfigureAwait(false);

            IsBusy = false;
        }

        private ICommand _ForceFreeMachineCommand;
        public ICommand ForceFreeMachineCommand
        {
            get => _ForceFreeMachineCommand;
            set => SetProperty(ref _ForceFreeMachineCommand, value);
        }

        public async void ForceFreeMachineCommandExecute()
        {
            IsBusy = true;
            if (!await CheckConnection().ConfigureAwait(false))
            {
                IsConnected = false;

                IsBusy = false;
                return;
            }

            Machine.IManageInterface manageInterface = _Machine.Manage;

            await manageInterface.ForceFree().ConfigureAwait(false);
            await LoadData().ConfigureAwait(false);

            IsBusy = false;
        }
        #endregion

        #region INavigationService
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {

        }

        public override async void OnNavigatedTo(INavigationParameters parameters)
        {
            _ID = parameters["id"] as string;

            await LoadData().ConfigureAwait(false);
        }
        #endregion
    }
}
