﻿using System;
using System.Threading.Tasks;
using System.Windows.Input;
using Borepin.Base;
using Borepin.Service.Versioning;
using Prism.Navigation;
using Xamarin.Forms;

namespace Borepin.PageModel
{
    public class MainPageModel : PageModelBase
    {
        #region Private Members
        private readonly IVersioningService _VersioningService;
        #endregion

        #region Constructors
        public MainPageModel(INavigationService navigationService, IVersioningService versioningService) : base(navigationService)
        {
            _VersioningService = versioningService;

            NavigateCommand = new Command<string>(NavigateCommandExecute);
        }
        #endregion

        #region LoadData
        public override Task LoadData()
        {
            CurrentVersion = _VersioningService.CurrentVersion;
            CurrentBuild = _VersioningService.CurrentBuild;

            return Task.CompletedTask;
        }
        #endregion

        #region Fields
        private string _CurrentVersion;
        public string CurrentVersion
        {
            get => _CurrentVersion;
            set => SetProperty(ref _CurrentVersion, value);
        }

        private string _CurrentBuild;
        public string CurrentBuild
        {
            get => _CurrentBuild;
            set => SetProperty(ref _CurrentBuild, value);
        }
        #endregion

        #region Commands
        private ICommand _NavigationCommand;
        public ICommand NavigateCommand 
        {
            get => _NavigationCommand;
            set => SetProperty(ref _NavigationCommand, value);
        }
        public async void NavigateCommandExecute(string view)
        {
            await _NavigationService.NavigateAsync($"NavigationPage/{ view }").ConfigureAwait(false);
        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {

        }

        public override async void OnNavigatedTo(INavigationParameters parameters)
        {
            await LoadData().ConfigureAwait(false);
        }
        #endregion
    }
}
