﻿using Borepin.ViewModel;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Linq;
using Borepin.Model;
using System.Windows.Input;
using Prism.Commands;
using Prism.Navigation;
using Borepin.Service.BFFH;
using Borepin.Base;

namespace Borepin.PageModel
{
    public class ServerListPageModel : PageModelBase
    {
        #region Private Fields
        private readonly IBFFHService _BFFHService;
        #endregion

        #region Constructors
        public ServerListPageModel(INavigationService navigationService, IBFFHService bffhService) : base(navigationService)
        {
            _BFFHService = bffhService;

            AddInstancesCommand = new DelegateCommand(AddInstancesCommandExecute);
            SelectInstanceCommand = new DelegateCommand<object>(SelectInstanceCommandExecute);
        }
        #endregion

        #region Data
        public override async Task LoadData()
        {
            IList<Connection> list = await _BFFHService.GetConnections().ConfigureAwait(false);
            if (_BFFHService.IsConnected)
            {
                ActiveConnection = new ServerListItemViewModel(_BFFHService.CurrentConnection);

                list.Remove(_BFFHService.CurrentConnection);

                HasActiveConnection = true;
            }
            else
            {
                HasActiveConnection = false;
            }

            ServerListItemViewModel_List = list.Select(x => new ServerListItemViewModel(x)).ToList();

            IsBusy = false;
        }
        #endregion

        #region Fields
        private IList<ServerListItemViewModel> _ServerListItemViewModel_List;
        public IList<ServerListItemViewModel> ServerListItemViewModel_List
        {
            get => _ServerListItemViewModel_List;
            set => SetProperty(ref _ServerListItemViewModel_List, value);
        }

        private ServerListItemViewModel _ActiveConnection;
        public ServerListItemViewModel ActiveConnection
        {
            get => _ActiveConnection;
            set => SetProperty(ref _ActiveConnection, value);
        }

        private bool _HasActiveConnection = false;
        public bool HasActiveConnection
        {
            get => _HasActiveConnection;
            set => SetProperty(ref _HasActiveConnection, value);
        }
        #endregion

        #region Commands
        private ICommand _SelectInstanceCommand;
        public ICommand SelectInstanceCommand
        {
            get => _SelectInstanceCommand;
            set => SetProperty(ref _SelectInstanceCommand, value);
        }
        public async void SelectInstanceCommandExecute(object obj)
        {
            ServerListItemViewModel viewmodel = obj as ServerListItemViewModel;

            NavigationParameters parameters = new NavigationParameters
            {
                { "instance", viewmodel.Instance },
            };

            await _NavigationService.NavigateAsync("ServerPage", parameters).ConfigureAwait(false);
        }

        private ICommand _AddInstancesCommand;
        public ICommand AddInstancesCommand
        {
            get => _AddInstancesCommand;
            set => SetProperty(ref _AddInstancesCommand, value);
        }
        public async void AddInstancesCommandExecute()
        {
            await _NavigationService.NavigateAsync("AddServerProcess_SelectServerPage").ConfigureAwait(false);
        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {

        }

        public override async void OnNavigatedTo(INavigationParameters parameters)
        {
            await LoadData().ConfigureAwait(false);
        }
        #endregion
    }
}
